import vapoursynth as vs
import multiprocessing
import os
from pathlib import Path

core = vs.core
core.num_threads = 0	# Use all logical CPU cores (threads)

## Load shared library of SVP ##
dir_curr = os.path.dirname(os.path.realpath(__file__))	# Path to directory this python file is contained in
core.std.LoadPlugin(dir_curr+"/libsvpflow1.so")
core.std.LoadPlugin(dir_curr+"/libsvpflow2.so")

clip = video_in

#####################
# Your SVP settings #
#####################
super_params     = "{scale:{up:0},gpu:0,rc:true}"	# CHANGE to "gpu:1" to use your "first GPU"
analyse_params   = "{block:{w:32,overlap:0},main:{search:{coarse:{satd:false,type:2,distance:-6,bad:{range:0}},type:2}},refine:[{thsad:250}]}"


## From original ##
src_fps     = container_fps if container_fps>0.1 else 23.976
stereo_type = 0


##########
# CHANGE #
##########

#== Notes ======
# GOAL: Interpolate src_fps with factor fps_factor
# BUT: src_fps * fps_factor <= goal_fps (else also frame drops)
# ->e.g.: goal_fps=144, src_fps=30 -> 1<=fps_factor<5

## CHANGE ##
goal_fps = 144	# <= refresh rate of your display

## CHANGE: Performance profiles ## according to clipResolution (Higher video resolution -> Lower goal_fps -> Lower fps_factor)
clipResolution = clip.width * clip.height

if clipResolution <= 2073600:		# <=FHD
	goal_fps = goal_fps
elif clipResolution <= 3686400:		# <=WQHD
	goal_fps = goal_fps
elif clipResolution <= 8847360:		# <=4K
	goal_fps = goal_fps * 3/4
elif clipResolution <= 35389440:	# <=8K
	goal_fps = goal_fps / 4
elif clipResolution > 35389440:		# >8K
	goal_fps = src_fps

fps_factor = int(goal_fps / src_fps)

#####################
# Your SVP settings #
#####################
# with changed num:"+str(fps_factor)+"

smoothfps_params = "{rate:{num:"+str(fps_factor)+",den:1},algo:2,scene:{adaptive:331}}"


########## BEGIN OF base.py ##########
# This file is a part of SmoothVideo Project (SVP) ver.4
# This is NOT the full Vapoursynth script, all used variables are defined via
# JScript code that generates the full script text.

def interpolate(clip):

	## Simplified code ##
	super   = core.svp1.Super(clip,super_params)
	vectors = core.svp1.Analyse(super["clip"],super["data"],clip,analyse_params)
	smooth  = core.svp2.SmoothFps(clip,super["clip"],super["data"],vectors["clip"],vectors["data"],smoothfps_params,src=clip,fps=src_fps)

	return smooth


if stereo_type == 1:
	lf = interpolate(core.std.CropRel(clip,0,(int)(clip.width/2),0,0))
	rf = interpolate(core.std.CropRel(clip,(int)(clip.width/2),0,0,0))
	smooth = core.std.StackHorizontal([lf, rf])
elif stereo_type == 2:
	lf = interpolate(core.std.CropRel(clip,0,0,0,(int)(clip.height/2)))
	rf = interpolate(core.std.CropRel(clip,0,0,(int)(clip.height/2),0))
	smooth = core.std.StackVertical([lf, rf])
else:
	smooth = interpolate(clip)
########### END OF base.py ###########

smooth.set_output()
